@@ -4,32 +4,32 @@ from decimal import Decimal
 
 from simpleeval import simple_eval
 
+from trytond.i18n import gettext
 from trytond.model import ModelView, ModelSQL, MatchMixin, fields, \
-    sequence_ordered
+    DeactivableMixin, sequence_ordered
 from trytond.tools import decistmt
-from trytond.pyson import If, Eval
 from trytond.transaction import Transaction
 from trytond.pool import Pool
-from trytond import backend
 
-__all__ = ['PriceList', 'PriceListLine']
+from .exceptions import FormulaError
 
 
-class PriceList(ModelSQL, ModelView):
+class PriceList(DeactivableMixin, ModelSQL, ModelView):
     'Price List'
     __name__ = 'product.price_list'
 
     name = fields.Char('Name', required=True, translate=True,
         help="The main identifier of the price list.")
-    company = fields.Many2One('company.company', 'Company', required=True,
-        select=True, domain=[
-            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
-                Eval('context', {}).get('company', -1)),
-            ],
-        help=("Make the price list belong to the company.\n"
-            "It defines the currency of the price list."))
+    company = fields.Many2One(
+        'company.company', "Company", required=True, select=True,
+        help="Make the price list belong to the company.\n"
+        "It defines the currency of the price list.")
     tax_included = fields.Boolean('Tax Included',
         help="Check if result's formula includes taxes.")
+    unit = fields.Selection([
+            ('product_default', "Product Default"),
+            ], "Unit", required=True,
+        help="The unit in which the quantity is expressed.")
     lines = fields.One2Many('product.price_list.line', 'price_list', 'Lines',
         help="Add price formulas for different criterias.")
 
@@ -41,31 +41,56 @@ class PriceList(ModelSQL, ModelView):
     def default_tax_included():
         return False
 
-    def get_context_formula(self, party, product, unit_price, quantity, uom):
+    @classmethod
+    def default_unit(cls):
+        return 'product_default'
+
+    def get_context_formula(self, party, product, unit_price, quantity, uom,
+            pattern=None):
+        if product:
+            cost_price = product.get_multivalue('cost_price') or Decimal('0')
+            list_price = product.list_price_used
+            if list_price is None:
+                list_price = unit_price
+        else:
+            cost_price = Decimal('0')
+            list_price = unit_price
         return {
             'names': {
                 'unit_price': unit_price,
+                'cost_price': cost_price,
+                'list_price': list_price,
                 },
             }
 
+    def get_uom(self, product):
+        return product.default_uom
+
     def compute(self, party, product, unit_price, quantity, uom,
             pattern=None):
         'Compute price based on price list of party'
 
         Uom = Pool().get('product.uom')
 
+        def parents(categories):
+            for category in categories:
+                while category:
+                    yield category
+                    category = category.parent
+
         if pattern is None:
             pattern = {}
 
         pattern = pattern.copy()
         if product:
-            pattern['categories'] = [c.id for c in product.categories_all]
+            pattern['categories'] = [
+                c.id for c in parents(product.categories_all)]
             pattern['product'] = product.id
         pattern['quantity'] = Uom.compute_qty(uom, quantity,
-            product.default_uom, round=False) if product else quantity
+            self.get_uom(product), round=False) if product else quantity
 
         context = self.get_context_formula(
-            party, product, unit_price, quantity, uom)
+            party, product, unit_price, quantity, uom, pattern=pattern)
         for line in self.lines:
             if line.match(pattern):
                 return line.get_unit_price(**context)
@@ -84,43 +109,25 @@ class PriceListLine(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
         help="Apply only to products of this category.")
     product = fields.Many2One('product.product', 'Product', ondelete='CASCADE',
         help="Apply only to this product.")
-    quantity = fields.Float('Quantity', digits=(16, Eval('unit_digits', 2)),
-            depends=['unit_digits'],
+    quantity = fields.Float(
+        'Quantity',
+        domain=['OR', ('quantity', '>=', 0), ('quantity', '=', None)],
         help="Apply only when quantity is greater.")
-    unit_digits = fields.Function(fields.Integer('Unit Digits'),
-        'on_change_with_unit_digits')
     formula = fields.Char('Formula', required=True,
         help=('Python expression that will be evaluated with:\n'
-            '- unit_price: the original unit_price'))
+            '- unit_price: the original unit_price\n'
+            '- cost_price: the cost price of the product\n'
+            '- list_price: the list price of the product'))
 
     @classmethod
     def __setup__(cls):
-        super(PriceListLine, cls).__setup__()
-        cls._order.insert(1, ('price_list', 'ASC'))
-        cls._error_messages.update({
-                'invalid_formula': ('Invalid formula "%(formula)s" in price '
-                    'list line "%(line)s" with exception "%(exception)s".'),
-                })
-
-    @classmethod
-    def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        table = TableHandler(cls, module_name)
-
-        super(PriceListLine, cls).__register__(module_name)
-
-        # Migration from 2.4: drop required on sequence
-        table.not_null_action('sequence', action='remove')
+        super().__setup__()
+        cls.__access__.add('price_list')
 
     @staticmethod
     def default_formula():
         return 'unit_price'
 
-    @fields.depends('product')
-    def on_change_with_unit_digits(self, name=None):
-        if self.product:
-            return self.product.default_uom.digits
-
     @classmethod
     def validate(cls, lines):
         super(PriceListLine, cls).validate(lines)
@@ -137,18 +144,18 @@ class PriceListLine(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
         try:
             if not isinstance(self.get_unit_price(**context), Decimal):
                 raise ValueError
-        except Exception, exception:
-            self.raise_user_error('invalid_formula', {
-                    'formula': self.formula,
-                    'line': self.rec_name,
-                    'exception': exception,
-                    })
+        except Exception as exception:
+            raise FormulaError(
+                gettext('product_price_list.msg_invalid_formula',
+                    formula=self.formula,
+                    line=self.rec_name,
+                    exception=exception)) from exception
 
     def match(self, pattern):
         if 'quantity' in pattern:
             pattern = pattern.copy()
             quantity = pattern.pop('quantity')
-            if self.quantity is not None and self.quantity > quantity:
+            if self.quantity is not None and self.quantity > abs(quantity):
                 return False
         if 'categories' in pattern:
             pattern = pattern.copy()
